@@ -22,7 +22,8 @@ static bool is_space(char ch)
 }
 
 QPDFTokenizer::QPDFTokenizer() :
-    pound_special_in_name(true)
+    pound_special_in_name(true),
+    allow_eof(false)
 {
     reset();
 }
@@ -34,6 +35,12 @@ QPDFTokenizer::allowPoundAnywhereInName()
     this->pound_special_in_name = false;
 }
 
+void
+QPDFTokenizer::allowEOF()
+{
+    this->allow_eof = true;
+}
+
 void
 QPDFTokenizer::reset()
 {
@@ -73,7 +80,7 @@ QPDFTokenizer::resolveLiteral()
                     num[0] = p[1];
                     num[1] = p[2];
                     num[2] = '\0';
-                    char ch = (char)(strtol(num, 0, 16));
+                    char ch = static_cast<char>(strtol(num, 0, 16));
                     if (ch == '\0')
                     {
                         type = tt_bad;
@@ -266,7 +273,7 @@ QPDFTokenizer::presentCharacter(char ch)
 	{
 	    // We've accumulated \ddd.  PDF Spec says to ignore
 	    // high-order overflow.
-	    val += (char) strtol(bs_num_register, 0, 8);
+	    val += static_cast<char>(strtol(bs_num_register, 0, 8));
 	    memset(bs_num_register, '\0', sizeof(bs_num_register));
 	    bs_num_count = 0;
 	}
@@ -392,7 +399,7 @@ QPDFTokenizer::presentCharacter(char ch)
 	    {
 		num[0] = val[i];
 		num[1] = val[i+1];
-		char nch = (char)(strtol(num, 0, 16));
+		char nch = static_cast<char>(strtol(num, 0, 16));
 		nval += nch;
 	    }
 	    val = nval;
@@ -441,9 +448,17 @@ QPDFTokenizer::presentEOF()
     }
     else if (state != st_token_ready)
     {
-        QTC::TC("qpdf", "QPDF_Tokenizer EOF reading token");
-	type = tt_bad;
-	error_message = "EOF while reading token";
+        QTC::TC("qpdf", "QPDF_Tokenizer EOF reading token",
+                this->allow_eof ? 1 : 0);
+        if (this->allow_eof)
+        {
+            type = tt_eof;
+        }
+        else
+        {
+            type = tt_bad;
+            error_message = "EOF while reading token";
+        }
     }
 
     state = st_token_ready;
@@ -496,7 +511,7 @@ QPDFTokenizer::readToken(PointerHolder<InputSource> input,
 	}
 	else
 	{
-	    if (is_space((unsigned char)ch) &&
+	    if (is_space(static_cast<unsigned char>(ch)) &&
 		(input->getLastOffset() == offset))
 	    {
 		++offset;
