@@ -34,16 +34,28 @@ QPDF_Array::unparse()
     return result;
 }
 
+QPDFObject::object_type_e
+QPDF_Array::getTypeCode() const
+{
+    return QPDFObject::ot_array;
+}
+
+char const*
+QPDF_Array::getTypeName() const
+{
+    return "array";
+}
+
 int
 QPDF_Array::getNItems() const
 {
-    return (int)this->items.size();
+    return this->items.size();
 }
 
 QPDFObjectHandle
 QPDF_Array::getItem(int n) const
 {
-    if ((n < 0) || (n >= (int)this->items.size()))
+    if ((n < 0) || (n >= static_cast<int>(this->items.size())))
     {
 	throw std::logic_error(
 	    "INTERNAL ERROR: bounds error accessing QPDF_Array element");
@@ -75,7 +87,7 @@ void
 QPDF_Array::insertItem(int at, QPDFObjectHandle const& item)
 {
     // As special case, also allow insert beyond the end
-    if ((at < 0) || (at > (int)this->items.size()))
+    if ((at < 0) || (at > static_cast<int>(this->items.size())))
     {
 	throw std::logic_error(
 	    "INTERNAL ERROR: bounds error accessing QPDF_Array element");
